# coding: utf-8
from abc import ABCMeta
from abc import abstractmethod
from urllib.parse import urljoin
import json

from m3 import ApplicationLogicException
from requests import HTTPError
from requests import Response
import requests


class AbstractApiClient(metaclass=ABCMeta):
    """Абстрактный клиент для api сервисов."""

    @property
    @abstractmethod
    def base_url(self):
        """Возвращает базовый url для подключения к сервису."""
        raise NotImplementedError()

    @staticmethod
    def _raise_for_status(response: Response):
        try:
            response.raise_for_status()
        except HTTPError:
            error = response.json()
            json_message = json.loads(error["message"])
            if isinstance(json_message, dict):
                message = "\n".join(
                    "\n".join(msg) if isinstance(msg, list) else msg
                    for msg in json_message.values()
                )
            else:
                message = json_message
            raise ApplicationLogicException(message=message)


class ReadOnlyMixin:
    """Миксин с операциями для чтения данных."""

    def get(self):
        records = requests.get(self.base_url)

        self._raise_for_status(records)
        return records.json()['rows']

    def get_pagination(self, filters: dict):
        records = requests.get(self.base_url, params=filters)
        self._raise_for_status(records)
        return records.json()

    def get_by(self, filters: dict):
        records = requests.get(self.base_url, params=filters)

        self._raise_for_status(records)
        return records.json()['rows']

    def get_one(self, pk: int):
        url = urljoin(self.base_url, str(pk))
        record = requests.get(url)

        self._raise_for_status(record)
        return record.json()


class DataChangeMixin:
    """Миксин для операций по изменению данных.

    Создание/изменение/удаление данных.
    """

    def create_one(self, data: dict):
        record = requests.post(self.base_url, json=data)

        self._raise_for_status(record)
        return record.json()

    def update_one(self, pk: int, data: dict):
        url = urljoin(self.base_url, str(pk))
        record = requests.patch(url, json=data)

        self._raise_for_status(record)
        return record.json()

    def delete_one(self, pk: int):
        url = urljoin(self.base_url, str(pk))
        record = requests.delete(url)

        self._raise_for_status(record)
        return record.json()

class ReadOnlyApiClient(ReadOnlyMixin, AbstractApiClient):
    """Клиент для работы с api сервисов только для чтения."""


class BaseAPIClient(DataChangeMixin, ReadOnlyMixin, AbstractApiClient):
    """Базовый клиент для работы с api сервисов."""
